-------------------------------------------------------------------------------
-- |> Frames
-- Creates and updates the addon frames and widgets
-------------------------------------------------------------------------------
-- |> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local InCombatLockdown = InCombatLockdown
local GetTime = GetTime

-------------------------------------------------------------------------------
-- |> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

-------------------------------------------------------------------------------
-- |> LOCAL FUNCTIONS
-------------------------------------------------------------------------------
--=> Adjust Scroll Offset
-------------------------------------------------------------------------------
local offsetMax
local offset
function weizPVP.AdustScrollOffset(self, value)
	if not InCombatLockdown() then
		offsetMax = weizPVP.ListFrame:GetHeight() - weizPVP.ScrollFrame:GetHeight()
		offset = 0
		if value > 0 then
			offset =
				weizPVP.ScrollFrame:GetVerticalScroll() - (weizPVP.Options.Bars.RowHeight + weizPVP.Options.Bars.VerticalSpacing)
		else
			offset =
				weizPVP.ScrollFrame:GetVerticalScroll() + (weizPVP.Options.Bars.RowHeight + weizPVP.Options.Bars.VerticalSpacing)
		end
		if offset < 0 then
			weizPVP.ScrollFrame:SetVerticalScroll(0)
		elseif offset > offsetMax then
			weizPVP.ScrollFrame:SetVerticalScroll(offsetMax)
		else
			weizPVP.ScrollFrame:SetVerticalScroll(offset)
		end
	end
end

--=> AlignFrameToPixels
-----------------------------------------------------------------------
local x, y
local fScale
function weizPVP:AlignFrameToPixels(frame)
	if not InCombatLockdown() then
		x, y = frame:GetCenter()
		if x and y then
			x, y = x * frame:GetEffectiveScale(), y * frame:GetEffectiveScale()
			fScale = frame:GetScale()
			x, y = x / fScale, y / fScale
			frame:SetPoint("CENTER", nil, "BOTTOMLEFT", weizPVP.ScaleCoords(x, y))
		end
	end
end

-------------------------------------------------------------------------------
-- |> CONFIGURE HEADER
-------------------------------------------------------------------------------

function weizPVP:CreateCoreFrames()
	--=> weizPVP.CoreFrame: SETTINGS
	-----------------------------------------------------------------------
	weizPVP.CoreFrame = CreateFrame("Frame", "weizPVP.CoreFrame", UIParent)
	weizPVP.CoreFrame:SetClampedToScreen(true)
	weizPVP.CoreFrame:SetClipsChildren(true)
	weizPVP.CoreFrame:SetFrameStrata("MEDIUM")
	weizPVP.CoreFrame:SetPoint("CENTER", UIParent)
	weizPVP.CoreFrame:SetSize(
		weizPVP.Options.Frames.Width,
		(weizPVP.Options.Frames.Height - weizPVP.Options.Bars.VerticalSpacing)
	)
	weizPVP.CoreFrame:SetMovable(true)
	weizPVP.CoreFrame:EnableMouse(true)
	weizPVP.CoreFrame:SetDontSavePosition(true)
	weizPVP.CoreFrame:SetBackdrop(
		{
			bgFile = SM:Fetch("background", weizPVP.Options.Frames.BackgroundTexture),
			tile = false
		}
	)
	weizPVP.CoreFrame:SetBackdropColor(unpack(weizPVP.Options.Frames.BackgroundColor))
	weizPVP.CoreFrame:SetResizable(true)
	weizPVP.CoreFrame:SetMinResize(180, weizPVP.Options.Bars.RowHeight)
	weizPVP.CoreFrame:SetMaxResize(500, weizPVP.Options.Frames.List.Height)
	weizPVP.CoreFrame:Show()
	weizPVP.CoreFrame.isLocked = false
	weizPVP.CoreFrame.isPinned = false

	--=> weizPVP.ScrollFrame: SETTINGS
	--> The frame that is a child of weizPVP.CoreFrame that will be scolling with content
	-----------------------------------------------------------------------
	weizPVP.ScrollFrame = CreateFrame("ScrollFrame", "weizPVP.ScrollFrame", weizPVP.CoreFrame, "weizPVPSecureScrollFrame")
	weizPVP.ScrollFrame:EnableMouseWheel(true)
	weizPVP.ScrollFrame:SetClipsChildren(true)
	weizPVP.ScrollFrame:SetPoint("TOPLEFT", weizPVP.CoreFrame, "TOPLEFT", weizPVP.Options.Frames.BorderSize, 0)
	weizPVP.ScrollFrame:SetPoint(
		"BOTTOMRIGHT",
		weizPVP.CoreFrame,
		"BOTTOMRIGHT",
		(-1 * weizPVP.Options.Frames.BorderSize),
		0
	)
	--> MOUSEWHEEL
	weizPVP.ScrollFrame:SetScript(
		"OnMouseWheel",
		function(_, value)
			weizPVP.AdustScrollOffset(self, value)
		end
	)
	--=> weizPVP.ListFrame: SETTINGS
	--> The frame that is a child of weizPVP.CoreFrame that will be scolling with content
	-----------------------------------------------------------------------
	weizPVP.ListFrame = CreateFrame("Frame", "weizPVP.ListFrame", weizPVP.ScrollFrame)
	weizPVP.ListFrame:SetPoint("CENTER", weizPVP.ScrollFrame, "CENTER", 0, 1)
	weizPVP.ListFrame:SetPoint("TOP", weizPVP.CoreFrame, "BOTTOM", 0, 1)
	weizPVP.ListFrame:SetHeight(weizPVP.Options.Frames.List.Height)
	weizPVP.ListFrame:SetWidth(weizPVP.ScrollFrame:GetWidth() - weizPVP.Options.Frames.BorderSize)
	weizPVP.ScrollFrame:SetScrollChild(weizPVP.ListFrame)

	--=> weizPVP.HeaderFrame: SETTINGS
	-----------------------------------------------------------------------
	weizPVP.HeaderFrame = CreateFrame("Frame", "weizPVP.HeaderFrame", UIParent)
	weizPVP.HeaderFrame:SetClampedToScreen(true)
	weizPVP.HeaderFrame:SetFrameStrata("MEDIUM")
	weizPVP.HeaderFrame:SetPoint("BOTTOMLEFT", weizPVP.CoreFrame, "TOPLEFT")
	weizPVP.HeaderFrame:SetPoint("BOTTOMRIGHT", weizPVP.CoreFrame, "TOPRIGHT")
	weizPVP.HeaderFrame:SetHeight(weizPVP.Options.Frames.Header.Height)
	weizPVP.HeaderFrame:SetDontSavePosition(true)
	weizPVP.HeaderFrame:EnableMouse(true)
	weizPVP.HeaderFrame:SetBackdrop(
		{
			bgFile = SM:Fetch("background", "weizPVP: Background"),
			tile = false,
			edgeFile = SM:Fetch("border", "weizPVP: Border"),
			edgeSize = 1,
			insets = {left = 0, right = 0, top = 0, bottom = 0}
		}
	)
	weizPVP.HeaderFrame:SetBackdropColor(0.01, 0.01, 0.01, 0.9)
	weizPVP.HeaderFrame:SetBackdropBorderColor(0, 0, 0, 1)

	--=> weizPVP.HeaderFrame: Scripts
	-----------------------------------------------------------------------
	--> ON MOUSE DOWN
	-- * Move core frame if window allows (unlocked/unpinned/not-clamped)
	local lastTry = 0
	local timeBetweenChatMessage = 5
	-------------------------------------------
	weizPVP.HeaderFrame:SetScript(
		"OnMouseDown",
		function(_, button)
			if button == "LeftButton" then
				if weizPVP.CoreFrame.isLocked == true then
					weizPVP:SetStatusBarMessage(" Window is |cffff3838LOCKED|r", true, 1)
					weizPVP.PlaySoundAlert(SM:Fetch("sound", "weizPVP: Error 1"))
					if lastTry + timeBetweenChatMessage < GetTime() then
						weizPVP:PrintAddonMessage("Window is |cffff3838LOCKED|r. Use Ctrl + Right click on the title bar to unlock it.")
						lastTry = GetTime()
					end
				elseif weizPVP.CoreFrame.isPinned == true then
					weizPVP:SetStatusBarMessage(" Window is |cFFF4564DPINNED|r", true, 1)
					weizPVP.PlaySoundAlert(SM:Fetch("sound", "weizPVP: Error 1"))
					if lastTry + timeBetweenChatMessage < GetTime() then
						weizPVP:PrintAddonMessage(" Window is |cffff3838PINNED|r. Right click the title bar to unpin.")
						lastTry = GetTime()
					end
				else
					weizPVP.CoreFrame:StartMoving()
					weizPVP.CoreFrame:SetClampRectInsets(
						0,
						0,
						weizPVP.Options.Frames.Header.Height + 1,
						-1 * (weizPVP.Options.Frames.Footer.Height)
					)
					weizPVP.CoreFrame.isMoving = true
				end
			elseif button == "RightButton" then
				if IsControlKeyDown() then
					weizPVP:ToggleLocked(not weizPVP.CoreFrame.isLocked)
				else
					weizPVP:TogglePinned()
				end
			end
		end
	)
	--> ON MOUSE UP
	-- * Stop frame
	-------------------------------------------
	weizPVP.HeaderFrame:SetScript(
		"OnMouseUp",
		function(_, button)
			if (weizPVP.CoreFrame.isMoving) then
				weizPVP.CoreFrame:StopMovingOrSizing()
				weizPVP.CoreFrame.isMoving = false
				weizPVP:SaveCoreFramePosition()
			end
		end
	)
	--> ON MOUSE ENTER
	-------------------------------------------
	weizPVP.HeaderFrame:SetScript(
		"OnEnter",
		function(_, button)
			UIFrameFadeIn(weizPVP.HeaderFrame.Highlight, 0.3, 0, 1)
		end
	)
	--> ON MOUSE LEAVE
	-------------------------------------------
	weizPVP.HeaderFrame:SetScript(
		"OnLeave",
		function(_, button)
			UIFrameFadeOut(weizPVP.HeaderFrame.Highlight, 0.2, 1, 0)
		end
	)
	--=> weizPVP.OuterIcons
	--> Frame that contains icons outside of the core frame
	-----------------------------------------------------------------------
	weizPVP.OuterIconsFrame = CreateFrame("Frame", "weizPVP.OuterIconsFrame", weizPVP.HeaderFrame)
	weizPVP.OuterIconsFrame:SetPoint("TOPLEFT", weizPVP.ScrollFrame, "TOPLEFT", -24, 0)
	weizPVP.OuterIconsFrame:SetPoint("BOTTOMRIGHT", weizPVP.ScrollFrame, "BOTTOMRIGHT", 0, 0)
	weizPVP.OuterIconsFrame:SetClipsChildren(true)

	--=> Header Frame Highlight
	-----------------------------------------------------------------------
	weizPVP.HeaderFrame.Highlight = CreateFrame("Frame", "weizPVP:HeaderFrame.Highlight", weizPVP.HeaderFrame)
	weizPVP.HeaderFrame.Highlight:SetAllPoints(true)
	weizPVP.HeaderFrame.Highlight.Texture = weizPVP.HeaderFrame.Highlight:CreateTexture(nil, "BACKGROUND")
	weizPVP.HeaderFrame.Highlight.Texture:SetAllPoints(true)
	weizPVP.HeaderFrame.Highlight.Texture:SetTexture("Interface\\AddOns\\weizPVP\\Textures\\white16x16.tga")
	weizPVP.HeaderFrame.Highlight.Texture:SetVertexColor(0.6, 0.6, 0.6, 1)
	weizPVP.HeaderFrame.Highlight.Texture:SetBlendMode("BLEND")
	weizPVP.HeaderFrame.Highlight:SetFrameLevel(1)
	weizPVP.HeaderFrame.Highlight:Hide()
	weizPVP.HeaderFrame.Highlight = weizPVP.HeaderFrame.Highlight

	--=> weizPVP.HeaderFrame.Text: SETTINGS
	-----------------------------------------------------------------------
	weizPVP.HeaderFrame.Text = weizPVP.HeaderFrame:CreateFontString(nil, "MEDIUM", nil)
	weizPVP.HeaderFrame.Text:SetPoint("LEFT", weizPVP.HeaderFrame, "LEFT", 8, 0)
	weizPVP.HeaderFrame.Text:SetFont(
		SM:Fetch("font", weizPVP.Options.Frames.Header.Font),
		weizPVP.Options.Frames.Header.FontSize,
		weizPVP.Options.Frames.Header.FontOutline
	)
	weizPVP.HeaderFrame.Text:SetJustifyH("LEFT")
	weizPVP.HeaderFrame.Text:SetJustifyV("MIDDLE")
	weizPVP.HeaderFrame.Text:SetTextColor(1, 1, 1, 1)
	weizPVP.HeaderFrame.Text:SetText("NEARBY |cff777777:|r |cFF26FF540|r")
	weizPVP.HeaderFrame.Text:SetScale(1)

	--=> weizPVP. Create Header Icons
	-----------------------------------------------------------------------
	weizPVP:CreateHeaderIcons()

	--=> weizPVP.FooterFrame: SETTINGS
	-----------------------------------------------------------------------
	weizPVP.FooterFrame = CreateFrame("Frame", "weizPVP.FooterFrame", weizPVP.HeaderFrame)
	weizPVP.FooterFrame:SetClampedToScreen(true)
	weizPVP.FooterFrame:SetFrameStrata("MEDIUM")
	weizPVP.FooterFrame:SetPoint("TOPLEFT", weizPVP.CoreFrame, "BOTTOMLEFT")
	weizPVP.FooterFrame:SetPoint("TOPRIGHT", weizPVP.CoreFrame, "BOTTOMRIGHT")
	weizPVP.FooterFrame:EnableMouse(false)
	weizPVP.FooterFrame:SetHeight(weizPVP.Options.Frames.Footer.Height)
	weizPVP.FooterFrame:SetBackdrop(
		{
			bgFile = SM:Fetch("background", "weizPVP: Background"),
			tile = false,
			edgeFile = SM:Fetch("border", "weizPVP: Border"),
			edgeSize = 1,
			insets = {left = 0, right = 0, top = 0, bottom = 0}
		}
	)
	weizPVP.FooterFrame:SetBackdropColor(0.01, 0.01, 0.01, 0.8)
	weizPVP.FooterFrame:SetBackdropBorderColor(0, 0, 0, 1)
	weizPVP.FooterFrame:SetScale(1)

	--=> weizPVP.FooterFrame.Text: SETTINGS
	-----------------------------------------------------------------------
	weizPVP.FooterFrame.Text = weizPVP.FooterFrame:CreateFontString(nil, "OVERLAY", nil)
	weizPVP.FooterFrame.Text:SetPoint("BOTTOMLEFT", weizPVP.FooterFrame, "BOTTOMLEFT", 2, 1)
	weizPVP.FooterFrame.Text:SetFont(SM:Fetch("font", weizPVP.Options.Frames.Footer.Font), 16, "OUTLINE")
	weizPVP.FooterFrame.Text:SetJustifyH("LEFT")
	weizPVP.FooterFrame.Text:SetJustifyV("MIDDLE")
	weizPVP.FooterFrame.Text:SetText("")

	weizPVP:CreateResizeGrip()

	if weizPVP.Options.Frames.Scale then
		weizPVP.CoreFrame:SetScale(weizPVP.Options.Frames.Scale)
		weizPVP.HeaderFrame:SetScale(weizPVP.Options.Frames.Scale)
	end
end

function weizPVP:SaveCoreFramePosition()
	weizPVP.Options.Frames.X = weizPVP.CoreFrame:GetLeft()
	weizPVP.Options.Frames.Y = weizPVP.CoreFrame:GetTop()
	weizPVP.Options.Frames.Width = weizPVP.FooterFrame:GetWidth()
	weizPVP.Options.Frames.Height = weizPVP.CoreFrame:GetHeight()
	weizPVP:SaveAddonData()
end

function weizPVP:SetCoreFramePosition(x, y, width, height)
	weizPVP.CoreFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
	weizPVP.HeaderFrame:ClearAllPoints()
	weizPVP.HeaderFrame:SetPoint("BOTTOMLEFT", weizPVP.CoreFrame, "TOPLEFT")
	weizPVP.HeaderFrame:SetPoint("BOTTOMRIGHT", weizPVP.CoreFrame, "TOPRIGHT")
	weizPVP.CoreFrame:SetWidth(width)
	weizPVP.CoreFrame:SetHeight(height)
	weizPVP:SaveCoreFramePosition()
end

local function WindowPin()
	weizPVP.CoreFrame:SetMovable(false)
	weizPVP.HeaderFrame:SetMovable(false)
	weizPVP.HeaderFrame:EnableMouse(true)
	weizPVP.CoreFrame.isLocked = false
	weizPVP.CoreFrame.isPinned = true
	weizPVP.Options.WindowPinned = true
	weizPVP:SetStatusBarMessage(" Window is now |cFFF4564DPINNED|r", true, 1)
end
local function WindowUnpin()
	weizPVP.CoreFrame:SetMovable(true)
	weizPVP.HeaderFrame:SetMovable(true)
	weizPVP.HeaderFrame:EnableMouse(true)
	weizPVP.CoreFrame.isLocked = false
	weizPVP.CoreFrame.isPinned = false
	weizPVP.Options.WindowPinned = false
	weizPVP:SetStatusBarMessage(" Window is now |cff42dcf4UNPINNED|r", true, 1)
end

function weizPVP:TogglePinned(value)
	if weizPVP.Options.WindowLocked == true then
		weizPVP:PrintAddonMessage(
			"Window is currently |cffff3838LOCKED|r. Use Ctrl + Right click on the title bar to unlock it."
		)
		return
	end

	if value ~= nil then
		--> non nil value; apply value
		if value == true then
			WindowPin()
		else
			WindowUnpin()
		end
	else
		--> nil value; toggle pinning
		if not weizPVP.Options.WindowPinned then
			WindowPin()
		else
			--> pin frame
			WindowUnpin()
		end
	end
	weizPVP:SaveAddonData()
	return
end

function weizPVP:ToggleLocked(value)
	if value == nil then
		return
	end

	if value == true then
		weizPVP.CoreFrame:SetMovable(false)
		weizPVP.HeaderFrame:SetMovable(false)
		weizPVP.CoreFrame.isLocked = true
		weizPVP:SetStatusBarMessage(" Window is now |cffff3838LOCKED|r", true, 1)
	else
		weizPVP.CoreFrame:SetMovable(true)
		weizPVP.HeaderFrame:SetMovable(true)
		weizPVP.CoreFrame.isLocked = false
		weizPVP.CoreFrame.isPinned = false
		weizPVP:SetStatusBarMessage(" Window is now |cff37ff37UNLOCKED|r", true, 1)
	end

	weizPVP.Options.WindowLocked = value
	--> Save Options
	weizPVP:SaveAddonData()
	return
end
