-------------------------------------------------------------------------------
-- |> Player Data Manager
-- Functions that help acquire and manage data involving the PlayerDB
-------------------------------------------------------------------------------
-- |> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local select = select
local UnitIsEnemy = UnitIsEnemy
local UnitIsPlayer = UnitIsPlayer
local GetUnitName = GetUnitName
local GetGuildInfo = GetGuildInfo
local UnitClass = UnitClass
local UnitRace = UnitRace
local UnitLevel = UnitLevel
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local GetTime = GetTime
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local strsub = strsub

-------------------------------------------------------------------------------
-- |> GLOBAL FUNCTIONS
-------------------------------------------------------------------------------
--=> UpdatePlayerActiveCache
-->  Updates data in the PlayerActiveCache and then calls
--> Sends data off the Lists to be processed
local newPlayerOnList
-----------------------------------------------------------------------
function weizPVP:UpdatePlayerActiveCache(name, health, stealth, dead, role, GUID)
	if not weizPVP.PlayerDB[name] then
		weizPVP.PlayerDB[name] = {}
	end

	if weizPVP.PlayerActiveCache[GUID] then
		newPlayerOnList = false
	else
		weizPVP.PlayerActiveCache[GUID] = {}
		newPlayerOnList = true
	end

	--> Update PlayerCache info
	-----------------------------------------
	if GUID then
		weizPVP.PlayerActiveCache[GUID].GUID = GUID
		--> NAME
		if name then
			weizPVP.PlayerActiveCache[GUID].Name = name
		elseif weizPVP.PlayerDB[name].Name ~= nil then
			weizPVP.PlayerActiveCache[GUID].Name = weizPVP.PlayerDB[name].Name
		end

		--> LEVEL
		weizPVP.PlayerActiveCache[GUID].Estimated = weizPVP.PlayerDB[name].Estimated
		if weizPVP.PlayerDB[name].Level ~= nil then
			weizPVP.PlayerActiveCache[GUID].Level = weizPVP.PlayerDB[name].Level
		elseif weizPVP.PlayerDB[name].Level ~= nil then
			weizPVP.PlayerActiveCache[GUID].Level = 0
			weizPVP.PlayerDB[name].Level = 0 --!!
		end

		--> CLASS
		if weizPVP.PlayerDB[name].Class ~= nil then
			weizPVP.PlayerActiveCache[GUID].Class = weizPVP.PlayerDB[name].Class
		else
			weizPVP.PlayerActiveCache[GUID].Class = "UNKNOWN"
			weizPVP.PlayerDB[name].Class = "UNKNOWN"
		end

		--> GUILD
		if weizPVP.PlayerDB[name].Guild ~= nil then
			weizPVP.PlayerActiveCache[GUID].Guild = weizPVP.PlayerDB[name].Guild
		else
			if weizPVP.PlayerActiveCache[GUID].Estimated == true then
				weizPVP.PlayerActiveCache[GUID].Guild = "?"
				weizPVP.PlayerDB[name].Guild = "?"
			else
				weizPVP.PlayerActiveCache[GUID].Guild = ""
				weizPVP.PlayerDB[name].Guild = ""
			end
		end

		--> HEALTH
		if health ~= nil then
			weizPVP.PlayerActiveCache[GUID].Health = health
		elseif weizPVP.PlayerActiveCache[GUID].Health == nil then
			weizPVP.PlayerActiveCache[GUID].Health = 100
		end

		--> STEALTH
		if stealth ~= nil then
			weizPVP.PlayerActiveCache[GUID].Stealth = stealth
		end

		--> DEAD
		if dead ~= nil then
			weizPVP.PlayerActiveCache[GUID].Dead = dead
			if dead == true then
				weizPVP.PlayerActiveCache[GUID].Health = 0
			end
		end

		--> ROLE
		if role and role ~= "UNKNOWN" then
			weizPVP.PlayerActiveCache[GUID].Role = role
			weizPVP.PlayerDB[name].Role = role
		end
	end

	--> Add player Data to lists
	weizPVP:AddPlayerDataToNearby(GUID, newPlayerOnList)
end

--=> AddNewPlayerDB
--> Adds ONLY NEW Players to DB
-----------------------------------------------------------------------
function weizPVP:AddNewPlayerDB(nameAndRealm, class, level, race, guild, estimated, role)
	if not weizPVP.PlayerDB[nameAndRealm] then
		weizPVP.PlayerDB[nameAndRealm] = {}
	end

	weizPVP.PlayerDB[nameAndRealm].Name = nameAndRealm
	if class then
		weizPVP.PlayerDB[nameAndRealm].Class = class
	end
	weizPVP.PlayerDB[nameAndRealm].Level = level or 0
	weizPVP.PlayerDB[nameAndRealm].Race = race or ""
	weizPVP.PlayerDB[nameAndRealm].Role = role or "UNKNOWN"
	if estimated == true and guild == nil then
		weizPVP.PlayerDB[nameAndRealm].Guild = ""
	else
		weizPVP.PlayerDB[nameAndRealm].Guild = guild
	end
	weizPVP.PlayerDB[nameAndRealm].Timestamp = GetTime()

	weizPVP.PlayerDB[nameAndRealm].Estimated = estimated or true

	return true
end

--=> Update Player DB
--> All new entries and updates to weizPVP.PlayerDB happen here.
local dataAdded
-----------------------------------------------------------------------
function weizPVP:UpdatePlayerDB(nameAndRealm, class, level, race, guild, estimated, role)
	if not weizPVP.PlayerDB[nameAndRealm] then --> NEW PLAYER check
		return
	end

	--> Update player
	if weizPVP.PlayerDB[nameAndRealm].Estimated == true then
		if weizPVP.PlayerDB[nameAndRealm].Level and level and weizPVP.PlayerDB[nameAndRealm].Level < level then
			weizPVP.PlayerDB[nameAndRealm].Level = level
		end

		if estimated == false then
			weizPVP.PlayerDB[nameAndRealm].Estimated = estimated
		end
	else
		--> not estimating, use the level from the db, if higher
		if weizPVP.PlayerDB[nameAndRealm] and level and weizPVP.PlayerDB[nameAndRealm].Level < level then
			weizPVP.PlayerDB[nameAndRealm].Level = level
		end
	end

	--> Get Role
	if role and role ~= "UNKNOWN" then
		weizPVP.PlayerDB[nameAndRealm].Role = role
	end
	--> Get Guild
	if guild ~= nil then
		weizPVP.PlayerDB[nameAndRealm].Guild = guild
	end

	weizPVP.PlayerDB[nameAndRealm].Timestamp = GetTime()

	dataAdded = true
	return dataAdded
end

--=> GET UNIT DATA
-----------------------------------------------------------------------
--> Gets data for events with a UNITID, such as 'target' or 'nameplate'
local Role
local Dead
local Health
local GUID
local timestamp = GetTime()
local name
local lastUpdatedUnit = {}
local UnitDataThrottle = 0.5
function weizPVP:GetUnitData(unit)
	--> Confirm Unit is a player that is an enemy or attackable
	if (UnitIsEnemy("player", unit) or weizPVP.Options.GLOBAL_DEBUG) and UnitIsPlayer(unit) then
		info = {}
		timestamp = GetTime()
		name = GetUnitName(unit, true)
		GUID = UnitGUID(unit) or nil
		Role = "UNKNOWN"
		--> Unit has valid name and GUID
		if name and GUID and name ~= "Unknown" then
			if not weizPVP.PlayerDB[name] then
				weizPVP.PlayerDB[name] = {}
				weizPVP.PlayerDB[name].Name = name
			end
			--> LAGMODE INACTIVE
			if weizPVP.Options.LagMode.Active == false then
				--> Only update non-health related info after 'UnitDataThrottle' time passes.
				if not lastUpdatedUnit[name] or lastUpdatedUnit[name] + UnitDataThrottle < timestamp then
					weizPVP.PlayerDB[name].Estimated = false
					weizPVP.PlayerDB[name].Guild = GetGuildInfo(unit)
					weizPVP.PlayerDB[name].Class = select(2, UnitClass(unit))
					weizPVP.PlayerDB[name].Race = select(1, UnitRace(unit))
					weizPVP.PlayerDB[name].Level = UnitLevel(unit)
					weizPVP.PlayerDB[name].Timestamp = timestamp
					lastUpdatedUnit[name] = timestamp
				end
			else
				weizPVP.PlayerDB[name].Timestamp = timestamp
			end
			--> Always update health/death info
			Dead = UnitIsDeadOrGhost(unit) or nil
			Health = (UnitHealth(unit) / UnitHealthMax(unit) * 100) or nil
			--> Update player data on bars
			weizPVP:UpdatePlayerActiveCache(name, Health, nil, Dead, nil, GUID)
		end
	end
end
